@@ -63,6 +63,10 @@ Cu.import('resource://app/modules/MailUtils.js');
 Cu.import('resource://app/modules/MailConsts.js');
 Cu.import('resource://app/modules/mailViewManager.js');
 
+/**
+ * List of keys not to export via installInto; values do not matter, we just
+ *  use true.
+ */
 const DO_NOT_EXPORT = {
   // magic globals
   MODULE_NAME: true, DO_NOT_EXPORT: true,
@@ -76,33 +80,40 @@ const DO_NOT_EXPORT = {
   MailUtils: true, MailViewManager: true,
   // internal setup functions
   setupModule: true, setupAccountStuff: true,
+  // we export this separately
+  teardownImporter: true,
   // internal setup flags
   initialized: false,
   // other libraries we use
+  testHelperModule: true,
   windowHelper: true
 };
 
+const EXPORT_VIA_GETTER_SETTER = {
+  // These should be getters and setters instead of direct property accesses so
+  // that setting them reflects across scopes.
+  mc: true,
+};
+
 var Application = Cc["@mozilla.org/steel/application;1"]
                     .getService(Ci.steelIApplication);
 
-var mainController = null;
-/** convenience shorthand for the mainController. */
+/** The controller for the main 3-pane window. */
 var mc;
 /** the index of the current 'other' tab */
 var otherTab;
 
 // These are pseudo-modules setup by setupModule:
-var messageGenerator;
-var messageModifier;
-var viewWrapperTestUtils;
+var testHelperModule;
 // (end pseudo-modules)
 
 var msgGen;
 
-var gLocalIncomingServer = null;
-var gLocalInboxFolder = null;
+var inboxFolder = null;
 
-var rootFolder = null;
+// logHelper exports
+var mark_action;
+var mark_failure;
 
 // the windowHelper module
 var windowHelper;
@@ -113,43 +124,54 @@ function setupModule() {
     return;
   initialized = true;
 
+  testHelperModule = {
+    Cc: Cc,
+    Ci: Ci,
+    Cu: Cu,
+    // fake some xpcshell stuff
+    _TEST_FILE: ["mozmill"],
+    do_throw: function(aMsg) {
+      throw new Error(aMsg);
+    },
+    do_check_eq: function() {},
+    do_check_neq: function() {},
+  };
+
   // The xpcshell test resources assume they are loaded into a single global
   //  namespace, so we need to help them out to maintain their delusion.
-  messageGenerator = load_via_src_path(
-    'mailnews/test/resources/messageGenerator.js');
-  messageModifier = load_via_src_path(
-    'mailnews/test/resources/messageModifier.js');
-  viewWrapperTestUtils = load_via_src_path(
-    'mail/base/test/unit/resources/viewWrapperTestUtils.js');
-  // desired global types...
-  viewWrapperTestUtils.SyntheticMessageSet =
-    messageModifier.SyntheticMessageSet;
-  viewWrapperTestUtils.do_throw = function(aMsg) {
-    throw new Error(aMsg);
-  };
-  // viewWrapperTestUtils wants a gMessageGenerator (and so do we)
-  msgGen = new messageGenerator.MessageGenerator();
-  viewWrapperTestUtils.gMessageGenerator = msgGen;
-  viewWrapperTestUtils.gMessageScenarioFactory =
-    new messageGenerator.MessageScenarioFactory(msgGen);
+  load_via_src_path('mailnews/test/resources/logHelper.js',
+                    testHelperModule);
+  mark_action = testHelperModule.mark_action;
+  mark_failure = testHelperModule.mark_failure;
+
+  load_via_src_path('mailnews/test/resources/asyncTestUtils.js',
+                    testHelperModule);
+  load_via_src_path('mailnews/test/resources/messageGenerator.js',
+                    testHelperModule);
+  load_via_src_path('mailnews/test/resources/messageModifier.js',
+                    testHelperModule);
+  load_via_src_path('mailnews/test/resources/messageInjection.js',
+                    testHelperModule);
+  load_via_src_path('mail/base/test/unit/resources/viewWrapperTestUtils.js',
+                    testHelperModule);
+
+  // messageInjection wants a gMessageGenerator (and so do we)
+  msgGen = new testHelperModule.MessageGenerator();
+  testHelperModule.gMessageGenerator = msgGen;
+  testHelperModule.gMessageScenarioFactory =
+    new testHelperModule.MessageScenarioFactory(msgGen);
 
   make_new_sets_in_folders = make_new_sets_in_folder =
-    viewWrapperTestUtils.make_new_sets_in_folders;
-  add_sets_to_folders = viewWrapperTestUtils.add_sets_to_folders;
-  create_virtual_folder = viewWrapperTestUtils.make_virtual_folder;
-
-  viewWrapperTestUtils.Ci = Ci;
-  viewWrapperTestUtils.Cu = Cu;
-  viewWrapperTestUtils.Cc = Cc;
+    testHelperModule.make_new_sets_in_folders;
+  add_sets_to_folders = testHelperModule.add_sets_to_folders;
 
   // use window-helper's augment_controller method to get our extra good stuff
   //  we need.
   windowHelper = collector.getModule('window-helpers');
-  mc = mainController = windowHelper.wait_for_existing_window("mail:3pane");
+  mc = windowHelper.wait_for_existing_window("mail:3pane");
   windowHelper.augment_controller(mc);
 
   setupAccountStuff();
-  viewWrapperTestUtils.gLocalIncomingServer = gLocalIncomingServer;
 }
 
 /**
@@ -160,34 +182,96 @@ function installInto(module) {
 
   // now copy everything into the module they provided to us...
   let us = collector.getModule('folder-display-helpers');
+  let self = this;
   for each (let [key, value] in Iterator(us)) {
-    if (!(key in DO_NOT_EXPORT) &&
-        key[0] != "_")
+    if (key in EXPORT_VIA_GETTER_SETTER) {
+      // The value of |key| changes between iterations, so it's important to
+      // capture the right key in a local variable.
+      let thisKey = key;
+      module.__defineGetter__(thisKey, function () self[thisKey]);
+      module.__defineSetter__(thisKey, function (aNewValue) {
+                               self[thisKey] = aNewValue;
+                             });
+    }
+    else if (!(key in DO_NOT_EXPORT) &&
+             key[0] != "_") {
       module[key] = value;
+    }
   }
+
+  // Export the teardown helper
+  let customTeardown = null;
+  // Mozmill uses __teardownModule__ to store what it thinks is the
+  // teardownModule function. Unfortunately, all this is figured out when the
+  // file is loaded, and we're evaluated much too late for that, so overwrite
+  // it.
+  if ("__teardownModule__" in module)
+    customTeardown = module.__teardownModule__;
+  module.__teardownModule__ = teardownImporter(customTeardown);
 }
 
 function setupAccountStuff() {
-  // Create a local account to work with folders.
-  // (Note this gives you an Outbox and Trash folder by default).
-  let acctMgr = Components.classes["@mozilla.org/messenger/account-manager;1"]
-                          .getService(Components.interfaces.nsIMsgAccountManager);
-  //acctMgr.createLocalMailAccount();
+  inboxFolder = testHelperModule.configure_message_injection({mode: "local"});
+}
 
-  gLocalIncomingServer = acctMgr.localFoldersServer;
+/**
+ * This returns a function that cleans up state in case any tests have failed,
+ * so that the chain of failures isn't propagated to the rest of the suite. We
+ * attempt to guarantee that after the teardown is executed:
+ * - exactly one 3-pane window is open, and its controller is assigned to |mc|
+ * - there are no other windows open
+ * - the 3-pane window has exactly one tab open -- the main 3-pane tab
+ * - the folder mode is set to All Folders
+ *
+ * @param [customTeardown] A custom teardown function, if it's already been
+ *     defined in a particular module. This will always be executed before any
+ *     cleanup we perform.
+ */
+function teardownImporter(customTeardown) {
+  let teardownModule = function teardownModule() {
+    if (customTeardown)
+      customTeardown();
 
-  rootFolder = gLocalIncomingServer.rootMsgFolder;
-  // Note: Inbox is not created automatically when there is no deferred server,
-  // so we need to create it.
-  gLocalInboxFolder = rootFolder.addSubfolder("Inbox");
-  // a local inbox should have a Mail flag!
-  gLocalInboxFolder.setFlag(Ci.nsMsgFolderFlags.Mail);
-  gLocalInboxFolder.setFlag(Ci.nsMsgFolderFlags.Inbox);
+    // - If there are no 3-pane windows open, open one.
+    let windowMediator = Cc["@mozilla.org/appshell/window-mediator;1"]
+                           .getService(Ci.nsIWindowMediator);
+    let mail3PaneWindow = windowMediator.getMostRecentWindow("mail:3pane");
+    if (!mail3PaneWindow) {
+      windowHelper.plan_for_new_window("mail:3pane");
+      let windowWatcher = Cc["@mozilla.org/embedcomp/window-watcher;1"]
+                            .getService(Ci.nsIWindowWatcher);
+      windowWatcher.openWindow(null,
+          "chrome://messenger/content/", "",
+          "all,chrome,dialog=no,status,toolbar", args);
+      mc = windowHelper.wait_for_new_window("mail:3pane");
+    }
+    else {
+      // - We might have a window open, but not be assigned to mc -- so if
+      //   mc.window.closed is true, look for a window to assign to mc.
+      if (!mc || mc.window.closed)
+        mc = windowHelper.wait_for_existing_window("mail:3pane");
+    }
 
-  // Force an initialization of the Inbox folder database.
-  var folderName = gLocalInboxFolder.prettiestName;
+    // Run through all open windows, closing any that aren't assigned to mc.
+    let enumerator = windowMediator.getEnumerator(null);
+    while (enumerator.hasMoreElements()) {
+      let win = enumerator.getNext();
+      if (win != mc.window)
+        win.close();
+    }
 
-  gLocalInboxFolder = rootFolder.getChildNamed("Inbox");
+    // At this point we should have exactly one window open.
+    // - Close all tabs other than the first one.
+    mc.tabmail.closeOtherTabs(mc.tabmail.tabInfo[0]);
+
+    // - Set the mode to All Folders.
+    if (mc.folderTreeView.mode != "all")
+      mc.folderTreeView.mode = "all";
+  };
+  // Another internal mozmill thing, again figured out too early for it to have
+  // a chance.
+  teardownModule.__name__ = "teardownModule";
+  return teardownModule;
 }
 
 /*
@@ -205,7 +289,7 @@ const FAST_INTERVAL = 100;
  * Create a folder and rebuild the folder tree view.
  */
 function create_folder(aFolderName) {
-  let folder = rootFolder.addSubfolder(aFolderName);
+  let folder = testHelperModule.make_empty_folder(aFolderName);
   mc.folderTreeView.mode = "all";
   return folder;
 }
@@ -215,7 +299,7 @@ function create_folder(aFolderName) {
  *  sure to rebuild the folder tree afterwards.
  */
 function create_virtual_folder() {
-  let folder = viewWrapperTestUtils.make_virtual_folder.apply(null, arguments);
+  let folder = testHelperModule.make_virtual_folder.apply(null, arguments);
   mc.folderTreeView.mode = "all";
   return folder;
 }
@@ -225,7 +309,31 @@ function create_virtual_folder() {
  * Create a thread with the specified number of messages in it.
  */
 function create_thread(aCount) {
-  return new viewWrapperTestUtils.SyntheticMessageSet(viewWrapperTestUtils.gMessageScenarioFactory.directReply(aCount));
+  return new testHelperModule.SyntheticMessageSet(
+    testHelperModule.gMessageScenarioFactory.directReply(aCount));
+}
+
+/**
+ * Create and return a SyntheticMessage object.
+ *
+ * @param {Object} aArgs An arguments object to be passed to
+ *                       MessageGenerator.makeMessage()
+ */
+ function create_message(aArgs) {
+  return msgGen.makeMessage(aArgs);
+}
+
+/**
+ * Add a SyntheticMessage to a folder.
+ *
+ * @param {SyntheticMessage} aMsg
+ * @param {Object} aFolder
+ */
+function add_message_to_folder(aFolder, aMsg) {
+  // should presumably use async_run here, but since setupAccountStuff is
+  // using a local store, it should be safe to assume synchronicity
+  add_sets_to_folders([aFolder],
+                      [new testHelperModule.SyntheticMessageSet([aMsg])]); 
 }
 
 /**
@@ -272,7 +380,13 @@ function be_in_folder(aFolder) {
 }
 
 /**
- * Create a new tab displaying a folder, making that tab the current tab.
+ * Create a new tab displaying a folder, making that tab the current tab. This
+ * does not wait for message completion, because it doesn't know whether a
+ * message display will be triggered. If you know that a message display will be
+ * triggered, you should follow this up with
+ * |wait_for_message_display_completion(mc, true)|. If you know that a blank
+ * pane should be displayed, you should follow this up with
+ * |wait_for_blank_content_pane()| instead.
  *
  * @return The tab info of the current tab (a more persistent identifier for
  *     tabs than the index, which will change as tabs open/close).
@@ -359,10 +473,19 @@ function open_selected_message_in_new_window() {
  * tab.
  *
  * @param aMsgHdr The message header to display.
+ * @param [aExpectNew3Pane] This should be set to true if it is expected that a
+ *                          new 3-pane window will be opened as a result of
+ *                          the API call.
+ *
  * @returns The currently selected tab, guaranteed to be a folder tab.
  */
-function display_message_in_folder_tab(aMsgHdr) {
+function display_message_in_folder_tab(aMsgHdr, aExpectNew3Pane) {
+  if (aExpectNew3Pane)
+    windowHelper.plan_for_new_window("mail:3pane");
   MailUtils.displayMessageInFolderTab(aMsgHdr);
+  if (aExpectNew3Pane)
+    mc = windowHelper.wait_for_new_window("mail:3pane");
+
   wait_for_message_display_completion(mc, true);
 
   // Make sure that the tab we're returning is a folder tab
@@ -373,25 +496,41 @@ function display_message_in_folder_tab(aMsgHdr) {
 }
 
 /**
- * Switch to another tab.  If no tab is specified, we switch to the 'other' tab.
- *  That is the last tab we used, most likely the tab that was current when we
- *  created this tab.
+ * Switch to another folder or message tab.  If no tab is specified, we switch
+ *  to the 'other' tab.  That is the last tab we used, most likely the tab that
+ *  was current when we created this tab.
  *
  * @param aNewTab Optional, index of the other tab to switch to.
  */
 function switch_tab(aNewTab) {
+  if (typeof aNewTab == "number")
+    aNewTab = mc.tabmail.tabInfo[aNewTab];
+
+  // If the new tab is the same as the current tab, none of the below applies.
+  // Get out now.
+  if (aNewTab == mc.tabmail.currentTabInfo)
+    return;
+
   // If we're still loading a message at this point, wait for that to finish
   wait_for_message_display_completion();
   let targetTab = (aNewTab != null) ? aNewTab : otherTab;
   // now the current tab will be the 'other' tab after we switch
   otherTab = mc.tabmail.currentTabInfo;
+
+  // If the target tab's folder display has a something selected and its message
+  // pane is visible, plan for a message display.
+  if (targetTab.messageDisplay.visible && targetTab.folderDisplay.selectedCount)
+    plan_for_message_display(targetTab);
+
   mc.tabmail.switchToTab(targetTab);
-  // if there is something selected, wait for display completion
-  if (mc.folderDisplay.selectedCount)
-    wait_for_message_display_completion();
-  // otherwise wait for the pane to end up blank
-  else
-    wait_for_blank_content_pane();
+  if (mc.messageDisplay.visible) {
+    // if there is something selected, wait for display completion
+    if (mc.folderDisplay.selectedCount)
+      wait_for_message_display_completion(mc, true);
+    // otherwise wait for the pane to end up blank
+    else
+      wait_for_blank_content_pane();
+  }
 }
 
 /**
@@ -469,18 +608,34 @@ function assert_tab_titled_from(aTab, aWhat) {
  *  current tab.
  */
 function close_tab(aTabToClose) {
+  if (typeof aTabToClose == "number")
+    aTabToClose = mc.tabmail.tabInfo[aTabToClose];
+
   // get the current tab count so we can make sure the tab actually opened.
   let preCount = mc.tabmail.tabContainer.childNodes.length;
 
+  // If we're closing the current tab, a message or summary might be displayed
+  // in the tab we'll select next.
+  let nextTab = null;
+  if (aTabToClose == mc.tabmail.currentTabInfo) {
+    let selectedIndex = mc.tabmail.tabContainer.selectedIndex;
+    let nextIndex = (selectedIndex == preCount - 1) ? selectedIndex - 1 :
+      selectedIndex + 1;
+    nextTab = mc.tabmail.tabInfo[nextIndex];
+    if (nextTab.messageDisplay.visible && nextTab.folderDisplay.selectedCount)
+      plan_for_message_display(nextTab);
+  }
+
   mc.tabmail.closeTab(aTabToClose);
 
   // if there is a message visible in the tab, make sure we wait for the load
-  if (mc.folderDisplay.selectedCount)
-    wait_for_message_display_completion(
-      mc, mc.messageDisplay.displayedMessage != null);
-  // otherwise wait for the pane to end up blank
-  else
-    wait_for_blank_content_pane();
+  if (nextTab && mc.messageDisplay.visible) {
+    if (mc.folderDisplay.selectedCount)
+      wait_for_message_display_completion(mc, true);
+    // otherwise wait for the pane to end up blank
+    else
+      wait_for_blank_content_pane();
+  }
 
   // check that the tab count decreased
   if (mc.tabmail.tabContainer.childNodes.length != preCount - 1)
@@ -510,6 +665,7 @@ function select_none(aController) {
   }
   controller.sleep('subject()',
                    NORMAL_TIMEOUT, FAST_INTERVAL, noMessageChecker);
+  wait_for_blank_content_pane(aController);
 }
 
 /**
@@ -709,6 +865,99 @@ function assert_row_visible(aViewIndex) {
                     "the thread pane, but isn't.");
 }
 
+/**
+ * Assert that the given folder mode is the current one.
+ */
+function assert_folder_mode(aMode) {
+  let actualMode = mc.folderTreeView.mode;
+  if (actualMode != aMode)
+    throw new Error("The folder mode should be " + aMode +
+                    ", but is actually " + actualMode);
+}
+
+/**
+ * Assert that the given folder is the child of the given parent in the folder
+ * tree view. aParent == null is equivalent to saying that the given folder
+ * should be a top-level folder.
+ */
+function assert_folder_child_in_view(aChild, aParent) {
+  let actualParent = mc.folderTreeView.getParentOfFolder(aChild);
+  if (actualParent != aParent)
+    throw new Error("Folder " + aChild.URI + " should be the child of " +
+                    (aParent && aParent.URI) +
+                    ", but is actually the child of " +
+                    (actualParent && actualParent.URI));
+
+}
+
+/**
+ * Assert that the given folder is in the current folder mode and is visible.
+ *
+ * @returns The index of the folder, if it is visible.
+ */
+function assert_folder_visible(aFolder) {
+  let folderIndex = mc.folderTreeView.getIndexOfFolder(aFolder);
+  if (folderIndex == null)
+    throw new Error("Folder: " + aFolder.URI + " should be visible, but isn't");
+
+  return folderIndex;
+}
+
+/**
+ * Assert that the given folder is either not in the current folder mode at all,
+ * or is not currently visible.
+ */
+function assert_folder_not_visible(aFolder) {
+  let folderIndex = mc.folderTreeView.getIndexOfFolder(aFolder);
+  if (folderIndex != null)
+    throw new Error("Folder: " + aFolder.URI +
+                    " should not be visible, but is");
+}
+
+/**
+ * Collapse a folder if it has children. This will throw if the folder itself is
+ * not visible in the folder view.
+ */
+function collapse_folder(aFolder) {
+  let folderIndex = assert_folder_visible(aFolder);
+  let folderFTVItem = mc.folderTreeView.getFTVItemForIndex(folderIndex);
+  if (folderFTVItem.open)
+    mc.folderTreeView.toggleOpenState(folderIndex);
+}
+
+/**
+ * Expand a folder if it has children. This will throw if the folder itself is
+ * not visible in the folder view.
+ */
+function expand_folder(aFolder) {
+  let folderIndex = assert_folder_visible(aFolder);
+  let folderFTVItem = mc.folderTreeView.getFTVItemForIndex(folderIndex);
+  if (!folderFTVItem.open)
+    mc.folderTreeView.toggleOpenState(folderIndex);
+}
+
+/**
+ * Assert that a folder is currently visible and collapsed. This will throw if
+ * either of the two is untrue.
+ */
+function assert_folder_collapsed(aFolder) {
+  let folderIndex = assert_folder_visible(aFolder);
+  let folderFTVItem = mc.folderTreeView.getFTVItemForIndex(folderIndex);
+  if (folderFTVItem.open)
+    throw new Error("Folder: " + aFolder.URI + " should be collapsed, but isn't");
+}
+
+/**
+ * Assert that a folder is currently visible and expanded. This will throw if
+ * either of the two is untrue.
+ */
+function assert_folder_expanded(aFolder) {
+  let folderIndex = assert_folder_visible(aFolder);
+  let folderFTVItem = mc.folderTreeView.getFTVItemForIndex(folderIndex);
+  if (!folderFTVItem.open)
+    throw new Error("Folder: " + aFolder.URI + " should be expanded, but isn't");
+}
+
 /**
  * Clear the selection in the folder tree view.
  */
@@ -798,6 +1047,32 @@ function middle_click_on_folder(aFolder) {
           viewIndex];
 }
 
+/**
+ * Get a reference to the smart folder with the given name.
+ *
+ * @param aFolderName The name of the smart folder (e.g. "Inbox").
+ * @returns An nsIMsgFolder representing the smart folder with the given name.
+ */
+function get_smart_folder_named(aFolderName) {
+  let acctMgr = Cc["@mozilla.org/messenger/account-manager;1"]
+                  .getService(Ci.nsIMsgAccountManager);
+  let smartServer = acctMgr.FindServer("nobody", "smart mailboxes", "none");
+  return smartServer.rootFolder.getChildNamed(aFolderName);
+}
+
+/**
+ * Get a reference to the smart folder with the given name.
+ *
+ * @param aFolderName The name of the smart folder (e.g. "Inbox").
+ * @returns An nsIMsgFolder representing the smart folder with the given name.
+ */
+function get_smart_folder_named(aFolderName) {
+  let acctMgr = Cc["@mozilla.org/messenger/account-manager;1"]
+                  .getService(Ci.nsIMsgAccountManager);
+  let smartServer = acctMgr.FindServer("nobody", "smart mailboxes", "none");
+  return smartServer.rootFolder.getChildNamed(aFolderName);
+}
+
 /**
  * Assuming the context popup is popped-up (via right_click_on_row), select
  *  the deletion option.  If the popup is not popped up, you are out of luck.
@@ -841,6 +1116,31 @@ function press_delete(aController) {
   wait_for_folder_events();
 }
 
+/**
+ * Archive the selected messages, and wait for it to complete.
+ *
+ * @param aController The controller in whose context to do this, defaults to
+ *     |mc| if omitted.
+ */
+function archive_selected_messages(aController) {
+  if (aController === undefined)
+    aController = mc;
+  let expectedCount = aController.dbView.rowCount - aController.dbView.numSelected;
+  aController.keypress(null, "a", {});
+
+  // Wait for the view rowCount to decrease by the number of selected messages.
+  let messagesDeletedFromView = function() {
+    return aController.dbView.rowCount == expectedCount;
+  };
+  controller.waitForEval('subject()',
+                         NORMAL_TIMEOUT,
+                         FAST_INTERVAL, messagesDeletedFromView);
+  // the above may return immediately, meaning the event queue might not get a
+  //  chance.  give it a chance now.
+  aController.sleep(0);
+}
+
+
 /**
  * Pretend we are pressing the Enter key, triggering opening selected messages.
  * Note that since we don't know where this is going to trigger a message load,
@@ -880,17 +1180,59 @@ function wait_for_all_messages_to_load(aController) {
 }
 
 /**
- * If  a message is in the process of loading, let it finish.  Otherwise we get
- *  horrible assertions like so:
+ * Call this before triggering a message display that you are going to wait for
+ *  using |wait_for_message_display_completion| where you are passing true for
+ *  the aLoadDemanded argument.  This ensures that if a message is already
+ *  displayed for the given controller that state is sufficiently cleaned up
+ *  so it doesn't trick us into thinking that there is no need to wait.
+ *
+ * @param [aControllerOrTab] optional controller or tab, defaulting to |mc|. If
+ *     the message display is going to be caused by a tab switch, a reference to
+ *     the tab to switch to should be passed in.
+ */
+function plan_for_message_display(aControllerOrTab) {
+  if (aControllerOrTab === undefined)
+    aControllerOrTab = mc;
+  // We're relying on duck typing here -- both controllers and tabs expose their
+  // message displays as the property |messageDisplay|.
+  aControllerOrTab.messageDisplay.messageLoaded = false;
+}
+
+/**
+ * If a message or summary is in the process of loading, let it finish;
+ *  optionally, be sure to wait for a load to happen (assuming
+ *  |plan_for_message_display| is used, modulo the conditions below.)
+ *
+ * This method is used defensively by a lot of other code in this file that is
+ *  realy not sure whether there might be a load in progress or not.  So by
+ *  default we only do something if there is obviously a message display in
+ *  progress.  Since some events may end up getting deferred due to script
+ *  blockers or the like, it is possible the event that triggers the display
+ *  may not have happened by the time you call this.  In that case, you should
+ *
+ *  1) pass true for aLoadDemanded, and
+ *  2) invoke |plan_for_message_display|
+ *
+ *  before triggering the event that will induce a message display.  Note that:
+ *  - You cannot do #2 if you are opening a new message window and can assume
+ *    that this will be the first message ever displayed in the window. This is
+ *    fine, because messageLoaded is initially false.
+ *  - You should not do #2 if you are opening a new folder or message tab. That
+ *    is because you'll affect the old tab's message display instead of the new
+ *    tab's display. Again, this is fine, because a new message display will be
+ *    created for the new tab, and messageLoaded will initially be false for it.
+ *
+ * If we didn't use this method defensively, we would get horrible assertions
+ *  like so:
  * ###!!! ASSERTION: Overwriting an existing document channel!
  *
- * @param aController optional controller, defaulting to |mc|.
- * @param aLoadDemanded optional indication that we expect and demand that a
- *     message be loaded.  If you call us before the message loading is
- *     initiated, you will need to pass true for this so that we don't see
- *     that a load hasn't started and assume none is required.  Defaults to
- *     false.  This relies on aController.messageDisplay.messageLoaded to
- *     be reliable; make sure it is false when entering this function.
+ *
+ * @param [aController] optional controller, defaulting to |mc|.
+ * @param [aLoadDemanded=false] Should we require that we wait for a message to
+ *     be loaded?  You should use this in conjunction with
+ *     |plan_for_message_display| as per the documentation above.  If you do
+ *     not pass true and there is no message load in process, this method will
+ *     return immediately.
  */
 function wait_for_message_display_completion(aController, aLoadDemanded) {
   if (aController === undefined)
@@ -910,8 +1252,10 @@ function wait_for_message_display_completion(aController, aLoadDemanded) {
   //  for us.
   let isLoadedChecker = function() {
     // If a load is demanded, first require that MessageDisplayWidget think
-    //  that the message is loaded.  Because the notification is imperfect,
-    //  this will strictly happen before the URL finishes running.
+    //  that the message is loaded.  Because the notification that sets the flag
+    //  happens when the message reader code gets told about the last attachment,
+    //  this will actually happen before the URL is finished running.  Luckily
+    //  we have the code below to try and deal with that.
     if (aLoadDemanded && !aController.messageDisplay.messageLoaded)
       return false;
 
@@ -1028,13 +1372,13 @@ function wait_for_folder_events() {
  *
  * @param aSynSets Either a single SyntheticMessageSet or a list of them.
  * @param aController Optional controller, which we get the folderDisplay
- *     property from.  If omitted, we use the mc (mainController).
+ *     property from.  If omitted, we use mc.
  */
 function assert_messages_in_view(aSynSets, aController) {
   if (aController === undefined)
     aController = mc;
-  viewWrapperTestUtils.verify_messages_in_view(aSynSets,
-                                               aController.folderDisplay.view);
+  testHelperModule.verify_messages_in_view(aSynSets,
+                                           aController.folderDisplay.view);
 }
 
 /**
@@ -1700,6 +2044,16 @@ let assert_no_folders_selected = assert_folders_selected_and_displayed;
 let assert_folder_selected_and_displayed =
     assert_folders_selected_and_displayed;
 
+/**
+ * Assert that there are the given number of rows (not including children of
+ * collapsed parents) in the folder tree view.
+ */
+function assert_folder_tree_view_row_count(aCount) {
+  if (mc.folderTreeView.rowCount != aCount)
+    throw new Error("The folder tree view's row count should be " + aCount +
+                    ", but is actually " + mc.folderTreeView.rowCount);
+}
+
 /**
  * Since indexOf does strict equality checking, we need this.
  */
@@ -1869,6 +2223,27 @@ function reset_context_menu_background_tabs() {
     prefBranch.clearUserPref("mail.tabs.loadInBackground");
 }
 
+/**
+ * Set the mail.close_message_window.on_delete pref.
+ *
+ * @param aPref true/false.
+ */
+function set_close_message_on_delete(aPref) {
+  let prefBranch = Cc["@mozilla.org/preferences-service;1"]
+                     .getService(Ci.nsIPrefService).getBranch(null);
+  prefBranch.setBoolPref("mail.close_message_window.on_delete", aPref);
+}
+
+/**
+ * Reset the mail.close_message_window.on_delete pref.
+ */
+function reset_close_message_on_delete() {
+  let prefBranch = Cc["@mozilla.org/preferences-service;1"]
+                     .getService(Ci.nsIPrefService).getBranch(null);
+  if (prefBranch.prefHasUserValue("mail.close_message_window.on_delete"))
+    prefBranch.clearUserPref("mail.close_message_window.on_delete");
+}
+
 /**
  * assert that the multimessage/thread summary view contains
  * the specified number of elements of the specified class.
@@ -1891,11 +2266,11 @@ function throw_and_dump_view_state(aMessage, aController) {
     aController = mc;
 
   dump("******** " + aMessage + "\n");
-  viewWrapperTestUtils.dump_view_state(aController.folderDisplay.view);
+  testHelperModule.dump_view_state(aController.folderDisplay.view);
   throw new Error(aMessage);
 }
 
-/** exported from viewWrapperTestUtils */
+/** exported from messageInjection */
 var make_new_sets_in_folders;
 var make_new_sets_in_folder;
 var add_sets_to_folders;
@@ -1907,8 +2282,18 @@ var add_sets_to_folders;
  *
  * @return An object that serves as the global scope for the loaded file.
  */
-function load_via_src_path(aPath) {
+function load_via_src_path(aPath, aModule) {
+  let loader = Cc["@mozilla.org/moz/jssubscript-loader;1"]
+                 .getService(Ci.mozIJSSubScriptLoader);
+  let ioService = Cc["@mozilla.org/network/io-service;1"]
+                    .getService(Ci.nsIIOService);
+
   let srcPath = os.abspath("../../../..",os.getFileForPath( __file__));
   let fullPath = os.abspath(aPath, os.getFileForPath(srcPath));
-  return frame.loadFile(fullPath, undefined);
+
+  let file = Cc["@mozilla.org/file/local;1"]
+               .createInstance(Ci.nsILocalFile);
+  file.initWithPath(fullPath);
+  let uri = ioService.newFileURI(file).spec;
+  loader.loadSubScript(uri, aModule);
 }
